#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import requests
import yaml
from tqdm import tqdm

from enum import Enum

from abxr.api_service import ApiService
from abxr.multipart import MultipartFileS3
from abxr.formats import DataOutputFormats

class Commands(Enum):
    LIST = "list"
    DETAILS = "details"
    VERSION_LIST = "versions"
    RELEASE_CHANNELS_LIST = "release_channels"
    RELEASE_CHANNEL_DETAILS = "release_channel_details"
    RELEASE_CHANNEL_SET_VERSION = "release_channel_set_version"
    UPLOAD = "upload"
    SHARE = "share"
    REVOKE_SHARE = "revoke"

class AppsService(ApiService):
    MAX_PARTS_PER_REQUEST = 4

    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def _initiate_upload(self, app_id, file_name):
        url = f'{self.base_url}/apps/{app_id}/versions'
        data = {'filename': file_name}
        
        response = requests.post(url, json=data, headers=self.headers)
        response.raise_for_status()
        
        return response.json()

    def _presigned_url(self, app_id, version_id, upload_id, key, part_numbers):
        url = f'{self.base_url}/apps/{app_id}/versions/{version_id}/pre-sign'
        data = {'key': key, 
                'uploadId': upload_id, 
                'partNumbers': part_numbers 
                }
        
        response = requests.post(url, json=data, headers=self.headers)
        response.raise_for_status()
        
        return response.json()

    def _complete_upload(self, app_id, version_id, upload_id, key, parts, version_name, release_notes):
        url = f'{self.base_url}/apps/{app_id}/versions/{version_id}/complete'
        data = {'key': key, 
                'uploadId': upload_id, 
                'parts': parts, 
                'versionName': version_name, 
                'releaseNotes': release_notes
                }
        
        response = requests.post(url, json=data, headers=self.headers)
        response.raise_for_status()
        
        return response.json()

    def upload_file(self, app_id, file_path, version, release_notes):
        print(f"Uploading {file_path} to app_id {app_id}...")

        file = MultipartFileS3(file_path)

        response = self._initiate_upload(app_id, file.file_name)

        upload_id = response['uploadId']
        key = response['key']
        version_id = response['versionId']

        part_numbers = list(range(1, file.get_part_numbers() + 1))

        uploaded_parts = []

        with tqdm(total=file.get_size(), unit='B', unit_scale=True, desc=f'Uploading {file.file_name}') as pbar:
            for i in range(0, len(part_numbers), self.MAX_PARTS_PER_REQUEST):
                part_numbers_slice = part_numbers[i:i + self.MAX_PARTS_PER_REQUEST]
                
                presigned_url_response = self._presigned_url(app_id, version_id, upload_id, key, part_numbers_slice)
                
                for item in presigned_url_response:
                    part_number = item['partNumber']
                    presigned_url = item['presignedUrl']

                    part = file.get_part(part_number)
                    response = requests.put(presigned_url, data=part)
                    response.raise_for_status()

                    uploaded_parts += [{'partNumber': part_number, 'eTag': response.headers['ETag']}]
                    pbar.update(len(part))
                
            complete_response = self._complete_upload(app_id, version_id, upload_id, key, uploaded_parts, version, release_notes)
            return complete_response
        
    def get_all_apps(self):
        url = f'{self.base_url}/apps?per_page=20'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        json = response.json()

        data = json['data']

        if json['links']:
            while json['links']['next']:
                response = requests.get(json['links']['next'], headers=self.headers)
                response.raise_for_status()
                json = response.json()

                data += json['data']

        return data
    
    def get_app_detail(self, app_id):
        url = f'{self.base_url}/apps/{app_id}'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()
    
    def get_all_versions_for_app(self, app_id):
        url = f'{self.base_url}/apps/{app_id}/versions'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        json = response.json()

        data = json['data']

        if json['links']:
            while json['links']['next']:
                response = requests.get(json['links']['next'], headers=self.headers)
                response.raise_for_status()
                json = response.json()

                data += json['data']

        return data
    
    def get_all_release_channels_for_app(self, app_id):
        url = f'{self.base_url}/apps/{app_id}/release-channels'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        json = response.json()

        data = json['data']

        if json['links']:
            while json['links']['next']:
                response = requests.get(json['links']['next'], headers=self.headers)
                response.raise_for_status()
                json = response.json()

                data += json['data']

        return data
    
    def get_release_channel_detail(self, app_id, release_channel_id):
        url = f'{self.base_url}/apps/{app_id}/release-channels/{release_channel_id}'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()
    
    def set_version_for_release_channel(self, app_id, release_channel_id, version_id):
        url = f'{self.base_url}/apps/{app_id}/release-channels/{release_channel_id}'

        data = {'versionId': version_id}

        response = requests.put(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()
    
    def share_app(self, app_id, release_channel_id, organization_slug):
        print(f"Sharing app_id {app_id} with release channel {release_channel_id} to org {organization_slug}...")

        url = f'{self.base_url}/apps/{app_id}/release-channels/{release_channel_id}/share'
        data = {'organizationSlug': organization_slug}

        response = requests.post(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()
    
    def revoke_shared_app(self, app_id, release_channel_id, organization_slug):
        print(f"Revoking Share of app_id {app_id} with release channel {release_channel_id} to org {organization_slug}...")

        url = f'{self.base_url}/apps/{app_id}/release-channels/{release_channel_id}/share'
        data = {'organizationSlug': organization_slug}

        response = requests.delete(url, json=data, headers=self.headers)
        response.raise_for_status()

        return response.json()

class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = AppsService(self.args.url, self.args.token)

    def run(self):
        if self.args.apps_command == Commands.LIST.value:            
            apps_list = self.service.get_all_apps()

            if self.args.format == DataOutputFormats.JSON.value:
                print(apps_list)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(apps_list))
            else:
                print("Invalid output format.")

        elif self.args.apps_command == Commands.DETAILS.value:
            app_detail = self.service.get_app_detail(self.args.app_id)

            if self.args.format == DataOutputFormats.JSON.value:
                print(app_detail)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(app_detail))
            else:
                print("Invalid output format.")

        elif self.args.apps_command == Commands.RELEASE_CHANNELS_LIST.value:
            release_channels = self.service.get_all_release_channels_for_app(self.args.app_id)

            if self.args.format == DataOutputFormats.JSON.value:
                print(release_channels)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(release_channels))
            else:
                print("Invalid output format.")

        elif self.args.apps_command == Commands.RELEASE_CHANNEL_DETAILS.value:
            release_channel_detail = self.service.get_release_channel_detail(self.args.app_id, self.args.release_channel_id)

            if self.args.format == DataOutputFormats.JSON.value:
                print(release_channel_detail)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(release_channel_detail))
            else:
                print("Invalid output format.")

        elif self.args.apps_command == Commands.VERSION_LIST.value:
            versions = self.service.get_all_versions_for_app(self.args.app_id)

            if self.args.format == DataOutputFormats.JSON.value:
                print(versions)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(versions))
            else:
                print("Invalid output format.")

        elif self.args.apps_command == Commands.RELEASE_CHANNEL_SET_VERSION.value:
            self.service.set_version_for_release_channel(self.args.app_id, self.args.release_channel_id, self.args.version_id)

        elif self.args.apps_command == Commands.UPLOAD.value:
            self.service.upload_file(self.args.app_id, self.args.filename, self.args.version, self.args.notes)

        elif self.args.apps_command == Commands.SHARE.value:
            self.service.share_app(self.args.app_id, self.args.release_channel_id, self.args.organization_slug)

        elif self.args.apps_command == Commands.REVOKE_SHARE.value:
            self.service.revoke_shared_app(self.args.app_id, self.args.release_channel_id, self.args.organization_slug)


#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import argparse
import os

from abxr.version import version
from abxr.formats import DataOutputFormats

from abxr.apps import Commands as AppCommands, CommandHandler as AppsCommandHandler
from abxr.files import Commands as FileCommands, CommandHandler as FilesCommandHandler

ABXR_API_URL = os.environ.get("ABXR_API_URL", "https://api.xrdm.app/api/v2")
ABXR_API_TOKEN = os.environ.get("ABXR_API_TOKEN") or os.environ.get("ARBORXR_ACCESS_TOKEN")


def main():
    parser = argparse.ArgumentParser(description=f'%(prog)s {version}')
    parser.add_argument("-u", "--url", help="API Base URL", type=str, default=ABXR_API_URL)
    parser.add_argument("-t", "--token", help="API Token", type=str, default=ABXR_API_TOKEN)
    parser.add_argument("-f", "--format", help="Data Output format", type=str, choices=[DataOutputFormats.JSON.value, DataOutputFormats.YAML.value], default=DataOutputFormats.YAML.value)
    parser.add_argument('-v', '--version', action='version', version=f'%(prog)s {version}')

    subparsers = parser.add_subparsers(dest="command", help="Sub-command help")

    apps_parser = subparsers.add_parser("apps", help="Apps command")
    apps_subparsers = apps_parser.add_subparsers(dest="apps_command", help="Apps command help")

    # List All Apps
    apps_list_parser = apps_subparsers.add_parser(AppCommands.LIST.value, help="List apps")

    # Detail of App
    app_detail_parser = apps_subparsers.add_parser(AppCommands.DETAILS.value, help="Detail of an app")
    app_detail_parser.add_argument("app_id", help="ID of the app", type=str)

    # Versions of an App
    versions_list_parser = apps_subparsers.add_parser(AppCommands.VERSION_LIST.value, help="List versions of an app")
    versions_list_parser.add_argument("app_id", help="ID of the app", type=str)


    # List Release Channels
    release_channels_list_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNELS_LIST.value, help="List release channels of an app")
    release_channels_list_parser.add_argument("app_id", help="ID of the app", type=str)

    # Detail of Release Channel
    release_channel_detail_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNEL_DETAILS.value, help="Detail of a release channel")
    release_channel_detail_parser.add_argument("app_id", help="ID of the app", type=str)
    release_channel_detail_parser.add_argument("--release_channel_id", help="ID of the release channel", type=str)

    # Set Version for Release Channel
    release_channel_set_version_parser = apps_subparsers.add_parser(AppCommands.RELEASE_CHANNEL_SET_VERSION.value, help="Set version for a release channel")
    release_channel_set_version_parser.add_argument("app_id", help="ID of the app", type=str)
    release_channel_set_version_parser.add_argument("--release_channel_id", help="ID of the release channel", type=str)
    release_channel_set_version_parser.add_argument("--version_id", help="ID of the version", type=str)

    # Upload and Create Version
    create_version_parser = apps_subparsers.add_parser(AppCommands.UPLOAD.value, help="Upload a new version of an app")
    create_version_parser.add_argument("app_id", help="ID of the app", type=str)
    create_version_parser.add_argument("filename", help="Local path of the APK to upload", type=str)
    create_version_parser.add_argument("-v", "--version", help="Version Number (APK can override this value)", type=str)
    create_version_parser.add_argument("-n", "--notes", help="Release Notes", type=str)

    # Sharing Apps
    share_parser = apps_subparsers.add_parser(AppCommands.SHARE.value, help="Share an app")
    share_parser.add_argument("app_id", help="ID of the app", type=str)
    share_parser.add_argument("--release_channel_id", help="ID of the release channel to share", type=str, required=True)
    share_parser.add_argument("--organization_slug", help="Slug of the organization to share with", type=str, required=True)

    # Revoke Sharing
    revoke_share_parser = apps_subparsers.add_parser(AppCommands.REVOKE_SHARE.value, help="Revoke sharing of an app")
    revoke_share_parser.add_argument("app_id", help="ID of the app", type=str)
    revoke_share_parser.add_argument("--release_channel_id", help="ID of the release channel to revoke", type=str, required=True)
    revoke_share_parser.add_argument("--organization_slug", help="Slug of the organization to revoke from", type=str, required=True)

    files_parser = subparsers.add_parser("files", help="Files command")
    files_subparsers = files_parser.add_subparsers(dest="files_command", help="Files command help")

    # List All Files
    files_list_parser = files_subparsers.add_parser(FileCommands.LIST.value, help="List files")

    # Detail of File
    file_detail_parser = files_subparsers.add_parser(FileCommands.DETAILS.value, help="Detail of a file")
    file_detail_parser.add_argument("file_id", help="ID of the file", type=str)

    # Upload a file
    upload_file_parser = files_subparsers.add_parser(FileCommands.UPLOAD.value, help="Upload a file")
    upload_file_parser.add_argument("filename", help="Local path of the file to upload", type=str)
    upload_file_parser.add_argument("-p", "--path", help="Path of the file on the device", type=str)

    args = parser.parse_args()

    if args.url is None:
        print("API URL is required")
        exit(1)

    if args.token is None:
        print("API Token is required. Please set the ABXR_API_TOKEN environment variable or use the --token command line param.")
        exit(1)

    if args.command == "apps":
        handler = AppsCommandHandler(args)
        handler.run()

    elif args.command == "files":
        handler = FilesCommandHandler(args)
        handler.run()
        

if __name__ == "__main__":
    main()


#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

from enum import Enum

class DataOutputFormats(Enum):
    JSON = "json"
    YAML = "yaml"


#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import os

class MultipartFileS3:
    MAX_PARTS = 1000
    MIN_PART_SIZE = 5 * 1024 * 1024
    MAX_PART_SIZE = 5 * 1024 * 1024 * 1024

    def __init__(self, file_path):
        self.file_path = file_path
        self.file_name = file_path.split('/')[-1]
        self.size = self.get_size()
        self.part_size = self.get_part_size()
        self.part_numbers = self.get_part_numbers()

    def get_size(self):
        return os.path.getsize(self.file_path)

    def get_part_size(self):
        part_size = self.size // self.MAX_PARTS
        return max(min(part_size, self.MAX_PART_SIZE), self.MIN_PART_SIZE)

    def get_part_numbers(self):
        return self.size // self.part_size + (1 if self.size % self.part_size > 0 else 0)

    def get_part(self, part_number):
        part_index = part_number - 1
        with open(self.file_path, 'rb') as file:
            file.seek(part_index * self.part_size)
            return file.read(self.part_size)


#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

class ApiService:
    def __init__(self, base_url, token):
        self.base_url = base_url
        self.headers = {
            'Authorization': f'Bearer {token}',
            'Content-Type': 'application/json'
        }




#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#
from setuptools import setup

setup(
    name = 'abxrcli',
    version = '0.9.4',
    packages = ['abxr'],
    entry_points = {
        'console_scripts': [
            'abxr-cli = abxr.cli:main',
        ]
    },
    install_requires = [
        'setuptools',
        'requests',
        'tqdm',
        'pyyaml'
    ],
    include_package_data=True,
)



#!/usr/bin/env python3
#
# Copyright (c) 2024-2025 ABXR Labs, Inc.
# Released under the MIT License. See LICENSE file for details.
#

import requests
import yaml
from tqdm import tqdm

from enum import Enum

from abxr.api_service import ApiService
from abxr.multipart import MultipartFileS3
from abxr.formats import DataOutputFormats

class Commands(Enum):
    LIST = "list"
    DETAILS = "details"
    UPLOAD = "upload"

class FilesService(ApiService):
    def __init__(self, base_url, token):
        super().__init__(base_url, token)

    def get_all_files(self):
        url = f'{self.base_url}/files?per_page=20'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        json = response.json()

        data = json['data']

        if json['links']:
            while json['links']['next']:
                response = requests.get(json['links']['next'], headers=self.headers)
                response.raise_for_status()
                json = response.json()

                data += json['data']

        return data
    
    def get_file_detail(self, file_id):
        url = f'{self.base_url}/files/{file_id}'

        response = requests.get(url, headers=self.headers)
        response.raise_for_status()

        return response.json()
    
class CommandHandler:
    def __init__(self, args):
        self.args = args
        self.service = FilesService(self.args.url, self.args.token)

    def run(self):
        if self.args.files_command == Commands.LIST.value:            
            files_list = self.service.get_all_files()

            if self.args.format == DataOutputFormats.JSON.value:
                print(files_list)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(files_list))
            else:
                print("Invalid output format.")

        elif self.args.files_command == Commands.DETAILS.value:
            file_detail = self.service.get_file_detail(self.args.file_id)

            if self.args.format == DataOutputFormats.JSON.value:
                print(file_detail)
            elif self.args.format == DataOutputFormats.YAML.value:
                print(yaml.dump(file_detail))
            else:
                print("Invalid output format.")

